@@ -122,6 +122,8 @@ typedef uint16 elf64_quarter;
 #define ELF_EM_960			19	/* Intel 80960 */
 #define ELF_EM_PPC			20	/* PowerPC */
 #define ELF_EM_PPC64			21	/* 64-bit PowerPC */
+#define ELF_EM_S390                     22	/* IBM S/390 */
+#define ELF_EM_SPU                      23	/* Sony/Toshiba/IBM SPU */
 
 #define ELF_EM_V800			36	/* NEC V800 series */
 #define ELF_EM_FR20			37	/* Fujitsu FR20 */
@@ -151,7 +153,9 @@ typedef uint16 elf64_quarter;
 #define ELF_EM_ST100			60	/* STMicroelectronics ST100 processor */
 #define ELF_EM_TINYJ			61	/* Advanced Logic Corp. TinyJ embedded processor */
 #define ELF_EM_X86_64			62	/* X86-64 (AMD Opteron) */
-
+#define ELF_EM_PDSP                     63	/* Sony DSP Processor */
+#define ELF_EM_PDP10                    64	/* Digital Equipment Corp. PDP-10 */
+#define ELF_EM_PDP11                    65	/* Digital Equipment Corp. PDP-11 */
 #define ELF_EM_FX66			66	/* Siemens FX66 microcontroller */
 #define ELF_EM_ST9PLUS			67	/* STMicroelectronics ST9+ 8/16 bit microcontroller */
 #define ELF_EM_ST7			68	/* STMicroelectronics ST7 8-bit microcontroller */
@@ -162,8 +166,81 @@ typedef uint16 elf64_quarter;
 #define ELF_EM_SVX			73	/* Silicon Graphics SVx */
 #define ELF_EM_ST19			74	/* STMicroelectronics ST19 8-bit microcontroller */
 #define ELF_EM_VAX			75	/* Digital VAX */
+#define ELF_EM_CRIS		 76	/* Axis Communications 32-bit embedded processor */
+#define ELF_EM_JAVELIN	 77	/* Infineon Technologies 32-bit embedded cpu */
+#define ELF_EM_FIREPATH	 78	/* Element 14 64-bit DSP processor */
+#define ELF_EM_ZSP		 79	/* LSI Logic's 16-bit DSP processor */
+#define ELF_EM_MMIX		 80	/* Donald Knuth's educational 64-bit processor */
+#define ELF_EM_HUANY	 81	/* Harvard's machine-independent format */
+#define ELF_EM_PRISM	 82	/* SiTera Prism */
+#define ELF_EM_AVR		 83	/* Atmel AVR 8-bit microcontroller */
+#define ELF_EM_FR30		 84	/* Fujitsu FR30 */
+#define ELF_EM_D10V		 85	/* Mitsubishi D10V */
+#define ELF_EM_D30V		 86	/* Mitsubishi D30V */
+#define ELF_EM_V850		 87	/* NEC v850 */
+#define ELF_EM_M32R		 88	/* Renesas M32R (formerly Mitsubishi M32R) */
+#define ELF_EM_MN10300	 89	/* Matsushita MN10300 */
+#define ELF_EM_MN10200	 90	/* Matsushita MN10200 */
+#define ELF_EM_PJ		 91	/* picoJava */
+#define ELF_EM_OPENRISC	 92	/* OpenRISC 32-bit embedded processor */
+#define ELF_EM_ARC_A5	 93	/* ARC Cores Tangent-A5 */
+#define ELF_EM_XTENSA	 94	/* Tensilica Xtensa Architecture */
+#define ELF_EM_VIDEOCORE	 95	/* Alphamosaic VideoCore processor */
+#define ELF_EM_TMM_GPP	 96	/* Thompson Multimedia General Purpose Processor */
+#define ELF_EM_NS32K	 97	/* National Semiconductor 32000 series */
+#define ELF_EM_TPC		 98	/* Tenor Network TPC processor */
+#define ELF_EM_SNP1K	 99	/* Trebia SNP 1000 processor */
+#define ELF_EM_ST200	100	/* STMicroelectronics ST200 microcontroller */
+#define ELF_EM_IP2K		101	/* Ubicom IP2022 micro controller */
+#define ELF_EM_MAX		102	/* MAX Processor */
+#define ELF_EM_CR		103	/* National Semiconductor CompactRISC */
+#define ELF_EM_F2MC16	104	/* Fujitsu F2MC16 */
+#define ELF_EM_MSP430	105	/* TI msp430 micro controller */
+#define ELF_EM_BLACKFIN	106	/* ADI Blackfin */
+#define ELF_EM_SE_C33	107	/* S1C33 Family of Seiko Epson processors */
+#define ELF_EM_SEP		108	/* Sharp embedded microprocessor */
+#define ELF_EM_ARCA		109	/* Arca RISC Microprocessor */
+#define ELF_EM_UNICORE	110	/* Microprocessor series from PKU-Unity Ltd. and MPRC of Peking University */
+#define ELF_EM_EXCESS	111	/* eXcess: 16/32/64-bit configurable embedded CPU */
+#define ELF_EM_DXP		112	/* Icera Semiconductor Inc. Deep Execution Processor */
+#define ELF_EM_ALTERA_NIOS2	113	/* Altera Nios II soft-core processor */
+#define ELF_EM_CRX		114	/* National Semiconductor CRX */
+#define ELF_EM_XGATE	115	/* Motorola XGATE embedded processor */
+#define ELF_EM_C166		116	/* Infineon C16x/XC16x processor */
+#define ELF_EM_M16C		117	/* Renesas M16C series microprocessors */
+#define ELF_EM_DSPIC30F	118	/* Microchip Technology dsPIC30F Digital Signal Controller */
+#define ELF_EM_CE		119	/* Freescale Communication Engine RISC core */
+#define ELF_EM_M32C		120	/* Renesas M32C series microprocessors */
+
+#define ELF_EM_TSK3000	131	/* Altium TSK3000 core */
+#define ELF_EM_RS08		132	/* Freescale RS08 embedded processor */
+
+#define ELF_EM_ECOG2	134	/* Cyan Technology eCOG2 microprocessor */
+#define ELF_EM_SCORE        135     /* Sunplus Score */
+#define ELF_EM_DSP24	136	/* New Japan Radio (NJR) 24-bit DSP Processor */
+#define ELF_EM_VIDEOCORE3	137	/* Broadcom VideoCore III processor */
+#define ELF_EM_LATTICEMICO32 138	/* RISC processor for Lattice FPGA architecture */
+#define ELF_EM_SE_C17	139	/* Seiko Epson C17 family */
+
+#define ELF_EM_MMDSP_PLUS	160	/* STMicroelectronics 64bit VLIW Data Signal Processor */
+#define ELF_EM_CYPRESS_M8C	161	/* Cypress M8C microprocessor */
+#define ELF_EM_R32C		162	/* Renesas R32C series microprocessors */
+#define ELF_EM_TRIMEDIA	163	/* NXP Semiconductors TriMedia architecture family */
+#define ELF_EM_QDSP6	164	/* QUALCOMM DSP6 Processor */
+#define ELF_EM_8051		165	/* Intel 8051 and variants */
+#define ELF_EM_STXP7X	166	/* STMicroelectronics STxP7x family */
+#define ELF_EM_NDS32	167	/* Andes Technology compact code size embedded RISC processor family */
+#define ELF_EM_ECOG1	168	/* Cyan Technology eCOG1X family */
+#define ELF_EM_ECOG1X	168	/* Cyan Technology eCOG1X family */
+#define ELF_EM_MAXQ30	169	/* Dallas Semiconductor MAXQ30 Core Micro-controllers */
+#define ELF_EM_XIMO16	170	/* New Japan Radio (NJR) 16-bit DSP Processor */
+#define ELF_EM_MANIK	171	/* M2000 Reconfigurable RISC Microprocessor */
+#define ELF_EM_CRAYNV2	172	/* Cray Inc. NV2 vector architecture */
+#define ELF_EM_RX		173	/* Renesas RX family */
+#define ELF_EM_METAG	174	/* Imagination Technologies META processor architecture */
+#define ELF_EM_MCST_ELBRUS	175	/* MCST Elbrus general purpose hardware architecture */
+#define ELF_EM_ECOG16	176	/* Cyan Technology eCOG16 family */
 
-#define ELF_EM_PJ			99	/* picoJava */
 
 /* e_version */
 #define ELF_EV_NONE			0
@@ -296,7 +373,7 @@ struct ELF_PROGRAM_HEADER32 {
 	elf32_word p_memsz;
 	elf32_word p_flags;
 	elf32_word p_align;
-};
+} PACKED;
 
 struct ELF_PROGRAM_HEADER64 {
 	elf64_half p_type;
@@ -313,7 +390,7 @@ struct ELF_NHEADER32 {
 	elf32_word n_descsz;
 	elf32_word n_namesz;
 	elf32_word n_type;
-};
+} PACKED;
 
 /*
  *	ELF symbol
@@ -345,7 +422,7 @@ struct ELF_SYMBOL32 {
 	elf_unsigned_char st_info;
 	elf_unsigned_char st_other;
 	elf32_half st_shndx;
-};
+} PACKED;
 
 struct ELF_SYMBOL64 {
 	elf64_half st_name;
@@ -354,7 +431,7 @@ struct ELF_SYMBOL64 {
 	elf64_quarter st_shndx;
 	elf64_word st_value;
 	elf64_word st_size;
-};
+} PACKED;
 
 /*
  *	ELF relocation
@@ -379,24 +456,24 @@ struct ELF_SYMBOL64 {
 struct ELF_REL32 {
 	elf32_addr	r_offset;
 	elf32_word	r_info;
-};
+} PACKED;
 
 struct ELF_REL64 {
 	elf64_addr	r_offset;
 	elf64_word	r_info;
-};
+} PACKED;
 
 struct ELF_RELA32 {
 	elf32_addr	r_offset;
 	elf32_word	r_info;
 	elf32_sword	r_addend;
-};
+} PACKED;
 
 struct ELF_RELA64 {
 	elf64_addr	r_offset;
 	elf64_word	r_info;
 	elf64_sword	r_addend;
-};
+} PACKED;
 
 extern byte ELF_HEADER_struct[];
 extern byte ELF_HEADER32_struct[];
