@@ -210,9 +210,9 @@ static char *_env_replace(char *str, unsigned int len)
 int sys_basename(char *result, const char *filename)
 {
 	// FIXME: use is_path_delim
-	char *slash1 = strrchr(filename, '/');
-	char *slash2 = strrchr(filename, '\\');
-	char *slash = (slash1 > slash2) ? slash1 : slash2;
+	const char *slash1 = strrchr(filename, '/');
+	const char *slash2 = strrchr(filename, '\\');
+	const char *slash = (slash1 > slash2) ? slash1 : slash2;
 	if (slash) {
 		int l = strlen(filename);
 		ht_strlcpy(result, slash+1, l-(slash-filename)+1);
@@ -395,14 +395,14 @@ int sys_common_canonicalize(char *result, const char *filename, const char *cwd,
 	return (k == 0) ? 0 : EINVAL;
 }
 
-char *sys_filename_suffix(const char *fn)
+const char *sys_filename_suffix(const char *fn)
 {
 	const char *s = NULL;
 	while (fn && *fn) {
 		if (sys_is_path_delim(*fn)) s = fn+1;
 		fn++;
 	}
-	char *p = s ? strrchr(s, '.') : NULL;
+	const char *p = s ? strrchr(s, '.') : NULL;
 	return p ? p+1 : NULL;
 }
 
